/*
 * nhpf.c - Hangul Print filter for Netscape (2.0 or later) PS save file, v1.1.
 * Lee yongjae, setup@shiva.snu.ac.kr, 1996.5.29.
 *
 * This program is the modified version of hpf v1.0.
 *
 * Copyright (c) 1996 by Lee yongjae, Narae in dept. of Computer Science
 * in Seoul National University, Korea.
 *
 * Narae is the name for Computer Science Society in dept. of
 * Computer Science in Seoul National University.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, 
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the copyright holder
 * not be used in advertising or publicity pertaining to distribution
 * of the software without specific, written prior permission.
 *
 * This software is provided "as is" without express or implied warranty.
 */
/*
 * This program make you can print hangul homepages with Netscape 2.0 or later,
 * in Unix with PostScript printer (or with Ghostscript).
 *
 * The filtered PS file is self-contained, because this program appends
 * a hangul font to the file. This means that you can print the file
 * with ANY(!) PostScript printer.
 *
 * Usage:
 *
 *   1. Save the Hangul homepage with "Save..." menu as "PostScript".
 *      (Do not save with "Print..." menu. "Print..." destroys texts.)
 *
 *   2. Filter the PS file through "nhpf", with redirection, such that:
 *  
 *        nhpf < original-ps-file > filtered-ps-file
 *
 *   3. Print the filtered PS file with PostScript printer.
 *
 * This program uses n-byte 3BeolSik hangul font or "n3f",
 * and the name of the font is "n3f-5", not a formal release.
 * 
 */

#include <stdio.h>
#include "ncode.h"

/* in "font.c" */
extern void print_hangul_font();

#define BUF_SIZE 20000
char buf[BUF_SIZE];

void remove_font_defs();
void print_font_defs();
void change_encoding();


void
main()
{
  remove_font_defs();
  print_font_defs();
  change_encoding();
}

void
change_encoding()
{
  unsigned char buf[100];
  unsigned char *p1, *p2;
  unsigned short k;
  int c, b0, b1, par_count = 0;
  
  while((c = getchar()) != EOF) {
    if (c == '\\') {
      if (scanf("%o", &b0) == 1) {
        if (scanf("\\%o", &b1) == 1) {
	  k = b0 * 256 +b1;
	  p1 = buf;
	  p2 = n_comb_to_n(n_ks_to_comb(k), buf);
	  while(p1 < p2)
	    printf("\\%o", (unsigned)*p1++);
        }
        else
          printf("\\%o",b0);
      }
      else {
        putchar(c);
        putchar(getchar());
      }
      continue;
    }

    putchar(c);

    if (c == '(')
      par_count++;
    else if (c == ')') {
      par_count--;
      if (par_count == 0) {
	gets(buf);  /* consum up " show\n" */
	puts(" nshow\n");
      }
    }
  }
}

void
remove_font_defs()
{
  while(fgets(buf, BUF_SIZE, stdin))
    if (buf[0] == '[')
      break;
    else
      fputs(buf, stdout);

  while(fgets(buf, BUF_SIZE, stdin))
    if (strncmp(buf, "/c", 2) == 0)
      break;

  fputs(buf, stdout);
  fgets(buf, BUF_SIZE, stdin);
  fputs(buf, stdout);

  while(fgets(buf, BUF_SIZE, stdin))
    if (strncmp(buf, "/f7", 3) == 0)
      break;
}

void
print_font_defs()
{
  puts("\n");
  print_hangul_font();
  puts("\n");

  puts("\
% define n3f-5-Slant font\n\
/n3f-5 findfont [ 1.0 0 0.277 1 0 0 ] makefont /n3f-5-Slant exch definefont\n\
\n\
/f0\n\
{ dup /Times-Roman findfont exch .9 mul scalefont /englishfont exch def\n\
  /n3f-5 findfont exch scalefont /hangulfont exch def\n\
} bind def\n\
/f1\n\
{ dup /Times-Bold findfont exch .9 mul scalefont /englishfont exch def\n\
  /n3f-5 findfont exch scalefont /hangulfont exch def\n\
} bind def\n\
/f2\n\
{ dup /Times-Italic findfont exch .9 mul scalefont /englishfont exch def\n\
  /n3f-5-Slant findfont exch scalefont /hangulfont exch def\n\
} bind def\n\
/f3\n\
{ dup /Times-BoldItalic findfont exch .9 mul scalefont /englishfont exch def\n\
  /n3f-5-Slant findfont exch scalefont /hangulfont exch def\n\
} bind def\n\
/f4\n\
{ dup /Courier findfont exch .9 mul scalefont /englishfont exch def\n\
  /n3f-5 findfont exch scalefont /hangulfont exch def\n\
} bind def\n\
/f5\n\
{ dup /Courier-Bold findfont exch .9 mul scalefont /englishfont exch def\n\
  /n3f-5 findfont exch scalefont /hangulfont exch def\n\
} bind def\n\
/f6\n\
{ dup /Courier-Italic findfont exch .9 mul scalefont /englishfont exch def\n\
  /n3f-5-Slant findfont exch scalefont /hangulfont exch def\n\
} bind def\n\
/f7\n\
{ dup /Courier-BoldItalic findfont exch .9 mul scalefont /englishfont exch def\n\
  /n3f-5-Slant findfont exch scalefont /hangulfont exch def\n\
} bind def\n\
\n\

/nshow  % (string) ncodeshow -\n\
{\n\
  -1 exch  % font str\n\
  { % font c\n\
    dup 128 lt { 0 } { 1 } ifelse dup 4 -1 roll ne  % c font boolean\n\
    { dup 0 eq { englishfont } { hangulfont } ifelse setfont } if  % c font\n\
    exch 1 string dup 0 4 -1 roll put show  % font\n\
  }\n\
  forall\n\
  pop\n\
}\n\
bind def\n\
");
}
